<?php
class BooksController extends AppController {

	var $name = 'Books';
	var $helpers = array('Html', 'Form', 'BookHelper', 'Ajax', 'Time');
	var $components = array('Filter', 'RequestHandler', 'Auth', 'Security');
	var $uses    = array('Book', 'User', 'Reservation', 'Pret', 'Exemplaire', 'Uregistration.Uruser');
	
	/**
	 * @var AuthComponent
	 */
	var $Auth;
	
	/**
	 * @var Book
	 */
	var $Book;
	
	/**
	 * @var User
	 */
	var $User;
	
	/**
	 * @var Reservation
	 */
	var $Reservation;
	
	/**
	 * @var Pret
	 */
	var $Pret;
	
	/**
	 * @var Exemplaire
	 */
	var $Exemplaire;
	
	/**
	 * Uruser model of Uregistration plugin
	 *
	 * @var model
	 */
	var $Uruser;
	/**
	 * @var SessionComponent
	 */
	var $Session;
	
	function beforeFilter () {
		parent::beforeFilter();
		$this->Auth->allow('index', 'view', 'new_arrivals','print_books','print_new','arraivals_hist','get_abso_book', 'numberOfExemplaires');

		$this->paginate = array(
		'Book'=> array(
          'limit' => 25,
          'order' => array(
              'Book.DateCreation' => 'desc',
          ))
        );
        $categs = Configure::read('opac.book.arrival_categs');
        if(!is_array($categs)) $categs = a();
        $this->categs = array_unique(am(array(1,9),$categs));
        
        $this->periods = array(
        	'all' => __('новинки',true),
        	'd' => __('за день',true),
			'wk'=> __('за неделю',true),
			'm' => __('за месяц',true),
        );
        $this->set('periods',$this->periods);
    }
    
    function arraivals_hist($year = '', $month = ''){
    	list($now_month,$now_year)=explode('-',date('m-Y'));
    	extract($this->params['named']);
    	$month_dif = ($year-$now_year)*12 + ($month-$now_month);
    	$paginate = array();
    	if(!empty($this->passedArgs['hist'])){
    		$month_dif = $this->passedArgs['hist'];
    		$page = coalesce($this->passedArgs['page'],0);
    		$paginate = array('page'=>$page);
    	}
    	$url = array(
    		'controller' => 'Books',
    		'action'=>'new_arrivals',
    		'hist'=>$month_dif,
    	);
    	if($month_dif >= 0){
    		$url = array(
    			'controller' => 'Books',
    			'action'=>'new_arrivals',
    			'm',
    		);
    	}
    	
	   	$this->redirect(am($url,$paginate));
    }

	function new_arrivals($period = 'all') {
		// задержка в днях между внесением книги и доступностью для читателя
        $delay = coalesce(Configure::read('opac.book.arrival_w8'),array('-14','-1','-1'));
  		// для истории поступлений:
  		$is_hist = false;
        if(!empty($this->passedArgs['hist'])){
        	$is_hist = true;
        	$period = 'm';
        	$hist = $this->passedArgs['hist'];
        	$delay[2] = $hist;
        	if($hist < -1) $delay[0] = '+0';
        }
  		
		$this->Book->arrivals = compact('period', 'delay');
				
		$this->set('title', __('Список новых поступлений', true));
		$this->Book->recursive = 1;
		
		if(!is_array($this->paginate['Book'])) {
			$this->paginate['Book'] = array();
		}
		
		$exemp_location_info = Configure::read('opac.book.exmp_loc_info');
		$exempContain = array('Statut');
		if (is_array($exemp_location_info))	
		  $exempContain = array_merge($exempContain, $exemp_location_info);
		
		$this->paginate['Book']['contain'] = array('NoticeZ', 'ExemplaireStat','Exemplaire'=>$exempContain,'Rating', 'Source'=>array('NoticeZ'));
		$this->paginate['Book']['joins']   = array(
			array('table' => 'view_210d',
				'alias' => 'view_210d',
				'with'  => array('NOEXPAND'),
				'type' => 'LEFT',
				'conditions' => array(
					'Book.IdNotice = view_210d.IdNotice',
					'Book.NotAvailableOpac = 0'
				)
			),
			array(
				'table' => 'view_notice_exemp_dates',
				'alias' => 'view_notice_exemp_dates',
				'type' => 'LEFT',
				'conditions' => array(
					'Book.IdNotice = view_notice_exemp_dates.IdNotice',
				),
			)
		);
		
		$year_min = intval(date('Y'))-5;
		$this->set('books', $this->paginate(null, array('Book.Categ'=>$this->categs, 'view_210d.Valeur > '.$year_min)));
		$this->set('arrivals_dates',$this->_get_arrivals_dates());

		// Делаем вид, что выводим без задержки
		//
		$delay = array('+0','-1','-1');
		if($is_hist){
			$delay[2] = $hist;
			$period = 'm';
			$this->set('delay',$hist);
		}
		$ids_print = $this->params['form']['ids_print'];
		if($ids_print !=''){
			$this->set('ids_print',$ids_print);
		}
		$this->Book->arrivals = compact('period', 'delay');
		$this->set('dates',$this->Book->findArrivalDates());
		
		$this->set('period',$is_hist ? 'hist' : $period);
		
		if($this->RequestHandler->isAjax()) { 
			$this->set('isAjax', true);
			$this->render('books_list_ajax'); 
		};
	}
	
	function _get_arrivals_dates(){
		$cache_key = $this->name.'._get_arrivals_dates';
		$arr = Cache::read($cache_key);
		if (!empty($arr)) return $arr;
		
		$filter = coalesce(Configure::read('opac.book.arrival_filter_by'), 'Notice.DateCreation');
		list($table, $field) = explode('.', $filter);
		
		$categ = implode(',',$this->categs);
		$year_min = intval(date('Y'))-5;
		
		$dates = $this->Book->query("select 
			CONVERT(nvarchar(7), t1.$field, 111) as date, count(DISTINCT t1.IdNotice)  as total
			from $table t1
			INNER JOIN Notice t2 ON t1.IdNotice = t2.IdNotice
			JOIN  view_210d t3 ON t1.IdNotice = t3.IdNotice
			where t2.Categ in ($categ) and t3.Valeur > $year_min
			group by CONVERT(nvarchar(7), t1.$field, 111) order by date");
		$dates = Set::combine($dates,'/0/date','/0/total');
		foreach ($dates as $date=>$total){
			if ($date == '') continue;
			list($year,$month) = explode('/',$date);
			$arr[$year][$month] = $total;
		}
		krsort($arr);
		Cache::write($cache_key, $arr, array('duration'=> '+1 hours'));
		return $arr;
	}
	
	function index($categ = null) {
		$this->set('title', __('Полный каталог изданий', true));
		$this->Book->recursive = 1;
		
		$categories = $this->Book->findCategories();
		
		if (intval($categ) == -1) {
			$conditions = array('Book.Categ NOT'=>Set::flatten(array_values($categories), ',')); 
		} else {
			$categ = Sanitize::paranoid($categ, array(','));
			$categ = explode(',',$categ);
			$conditions = array('Book.Categ'=>$categ); 
			
			$this->Book->bindModel(array('hasOne'=>array('PretHisto'=>array(),
			'PretHisto'=>array(
			'foreignKey'=>false,
			'conditions'=>array('PretHisto.notice_id = Book.IdNotice')),
			)
			), false);
		}		
		$conditions['Book.NotAvailableOpac'] = 0;
		$this->paginate['Book']['contain'] = array('NoticeZ', 'ExemplaireStat','Exemplaire'=>a('Statut','Annexe','Bibliotheque'),'Rating');
		$books = $this->paginate(null, $conditions);
		$isAjax = $this->RequestHandler->isAjax();
		$page = (int)$this->params['named']['page'];
		$this->set(compact('books', 'categories', 'page', 'isAjax'));
	}
	
	function view($id = null) {
		if (!$id) {
			$this->Session->setFlash(__('Неправильная книга', true));
			$this->redirect(array('action'=>'index'));
		}
				
		$this->set('book', $this->paginate($this->Book, $id));
	}
	
	/**
	 * Бронирование издания
	 *
	 * @param integer $id = IdNotice, идентификатор издания (описания)
	 */
	function reserve($id = null, $source = 'default') {
		$isAjax = $this->RequestHandler->isAjax();
		$defaults = array('IdAnnexe' => null);
		$params = array_merge($defaults, $this->params['named']);
				
		if ($isAjax) {
			$this->layout = 'ajax';
			$this->Book->contain(array('Exemplaire'=>array('Annexe','Bibliotheque'))); // optimization
		} else {
			$this->Book->contain(array('Exemplaire'=>array('Annexe','Bibliotheque'), 'NoticeZ'));
		}
		
		if($source != 'default'){
			$this->setErrorFlash(__('Нельзя забронировать документ из другой базы.', true));
			return;
		}
		
		if(!$id){
			$id = intval($params['IdNotice']);
			$source = $params['Source'] ? $params['Source'] : $source ;
		}
		if (!$id) {
			$this->Session->setFlash(__('Не задано издание.', true));
			return;
		}
		if(!$this->isSourceExist($source)){
			$this->Session->setFlash(__('Не задано место.', true));
			return;
		}
		$this->Book->recursive = 2;
		$this->Book->setDataSource($source);
		$book = $this->Book->find('first', array('conditions'=>array($this->Book->primaryKey => $id)));
		
		/** @var string $possibleAnnexe - выражение XPath для определения филиалов, для которых разрешена выдача **/
		$validAnnexe = '/Exemplaire/Annexe[ResaEnable=1]';
		
		/** @var array $annexes - массив для выбора филиала **/
		$annexes = Set::combine($book, $validAnnexe.'/IdAnnexe', $validAnnexe.'/Libelle');
		$exemps_by_annexe = Set::combine($book, '/Exemplaire/IdExemplaire', '/Exemplaire/Annexe/ResaEnable','/Exemplaire/Annexe/IdAnnexe');
		// массив филиалов где достуно только для чтения
		$annexesReadonly = array();
		$max_reserv = Configure::read('opac.book.max_in_lib');
		foreach ($exemps_by_annexe as $idAnnexe=>$exemps){
			if($max_reserv == 0) continue;
			if(array_sum($exemps) <= $max_reserv && is_string($annexes[$idAnnexe])){
				$annexesReadonly[$idAnnexe] = $annexes[$idAnnexe];
				unset($annexes[$idAnnexe]);
			}
		}
		
		$this->pageTitle = __('Бронирование экземпляра', true);
		$this->set(compact('book','annexes','annexesReadonly'));
		//$this->set('show_details', false && !$isAjax);
		$this->set('show_dubug_pre',  false && !$isAjax);
		$this->set('IdAnnexe', intval($params['IdAnnexe']));
	}
	
	function do_reservation($id, $source = 'default') {	
		$isAjax = $this->RequestHandler->isAjax();
		if ($isAjax) $this->layout = 'ajax';

		if (empty($this->data)) {
			$this->setErrorFlash(__('Недостаточно данных для бронирования.', true));
			return;
		}
		
		//$this->Reservation->setDataSource($source);
		//$this->Exemplaire->setDataSource($source);
		if($source != 'default'){
			$this->setErrorFlash(__('Нельзя забронировать документ из другой базы.', true));
			return;
		}
		
		// Дополняем данные для бронирования данными о книге и пользователе
		// $this->data['Reservation']['IdAnnexe'] from POST
		
		$this->data['Reservation']['IdNotice'] = $id;
		$this->data['Reservation']['Source'] = $source;
		$this->data['Reservation']['IdUsager'] = $this->Auth->user('IdUsager');
		
		if (!isset($this->data['Reservation']['IdAnnexe'])) {
			$this->setErrorFlash(__('Не задан филиал.', true));
			return;
		}
		if(!$this->isSourceExist($source)){
			$this->setErrorFlash(__('Не задана база.', true));
			return;
		}
		
		$IdAnnexe = intval($this->data['Reservation']['IdAnnexe']);
		$IdUsager = $this->Auth->user('IdUsager');
		$IdNotice = intval($this->data['Reservation']['IdNotice']);
		
		// Проверка подписок пользователя
		if (!$this->_checkUserSouscriptions($IdUsager, $IdAnnexe)) return false;
		
		// Проверка запретов и задолженностей пользователя
		if (!$this->_checkUserNoPenalsNoHoldOver($IdUsager, null)) return false;

		// Проверка максимального числа бронирований для читателя
		if (!$this->_checkMaxReservations($IdUsager)) return false;
				
		// Проверка, выдан или забронирован уже этим читателем данный документ
		if (!$this->_checkNotReservedOrPreted($IdUsager, $IdNotice)) return false;
		
		// @todo check availibility of exemplaires, and do reservation for first availible (use autoinc)
		$ex = $this->Exemplaire->alias;
		$fields = array("$ex.IdExemplaire", "$ex.CodbarE");
		$contain = array('Statut');
		$conditions = array("$ex.IdNotice"=>$IdNotice, "$ex.IdAnnexe"=>$IdAnnexe,
		  "$ex.IdExemplaire NOT IN (SELECT IdExemplaire FROM EXEMPLAIRE_RESA WHERE IdReservation IS NOT NULL)",
		  "$ex.IdExemplaire NOT IN (SELECT IdExemplaire FROM PRET)",
		  //"Statut.Pretable"=>1, 
		  "Statut.Reservable"=>1
		  );
		  
		$exemp_availible = $this->Exemplaire->find('all', compact('fields','contain','conditions'));
		if (empty($exemp_availible)) {
			$this->setErrorFlash(__('Невозможно забронировать документ: нет доступных для бронирования экземпляров',true));
			return;
		}
		
		$AddReservationReq = 
		"DECLARE @IdReservation INT
			UPDATE COMPTEUR SET ValCompteur = ValCompteur + 1 WHERE IdCompteur = 'RESERVATION'
			SELECT @IdReservation = ValCompteur FROM COMPTEUR WHERE IdCompteur = 'RESERVATION'
			INSERT RESERVATION (IdReservation, IdUsager, IdBibliotheque, IdAnnexe, DateResa, HeureResa, IdNotice, Attribuee)
			VALUES (@IdReservation, ".$IdUsager.", NULL, ".$IdAnnexe.", convert(char(8), getdate(), 112), convert(char(8), getdate(), 108), ".$IdNotice.", convert(char(8), getdate(), 112))";
		
		if(in_array(LIBER_PLUGIN_UREGISTRATION, Configure::read(LIBER_PLUGINS))){
			$AddReservationReq = $this->_addReservationToAutorecordArmoire($AddReservationReq, $IdNotice, $IdAnnexe, $IdUsager);
		}
		
		$res = $this->Reservation->query($AddReservationReq);
		$res = $this->Reservation->getAffectedRows();
		$this->set('reservation', $res);
		
		if ($res > 0) {
			$this->setGreenFlash(__('Экземпляр успешно забронирован.', true));
			return ;
			//$this->redirect(array('action'=>'index'));
		} else {
			$this->setErrorFlash(__('Не удалось забронировать документ. Пожалуйста, попробуйте в другой раз.', true));
			$this->set('success', false);
			return ;
		}
		
		$this->set('success', false);
		/** @todo использовать Attribuee - дату-время выполнения заказа (заказ на бронеполке), 
	 	 *  Attente - дату-время когда заказ был принят к выполнению (распечатан),
		 */
	}
	
	function print_books($ids = null, $source = 'default'){
		$this->layout = 'ajax';
		
		if(!$ids){// если прибыл список из другого места
			$sources = $this->params['form'];
		}
		if(!$sources){
			$sources = array($source=>$ids);
		}
		
		$res = '';
		$this->loadModel('RemoteNotice');
		$data = (array)$this->params['url'];

		if ($this->params['requested']) {
			$data = (array)$this->params['named'];
		}

		$this->set('sources', $sources);
	}
	
	function get_abso_book($id, $source = 'default', $row=false){
		$row = ($row || $this->params['named']['row']);
		$this->layout = 'ajax';
		$res = '';
		$this->loadModel('RemoteNotice');
		$data = (array)$this->params['url'];
		
		if(!$this->isSourceExist($source)) continue;

		$opac_config = Configure::read('opac_sources');
		$url = str_replace('ISBD_','ISBDList_',$opac_config->GetParam($source, 'ISBD_URL'));
		$data['FicLog'] = $opac_config->GetParam($source, 'fic');
		unset($data['url']);
		$data['IdSession'] = 'OPAC';
		$data['IdLangue'] = $this->IdLangue;
		$data['ids'] = (is_array($id) ? implode(',', $id) : $id);

		$res = $this->RemoteNotice->request('post', array('data' => $data, 'url' => $url, 'options'=>Configure::read('opac.remoteRequestOptions')));
		$this->set('res', $res);
		$this->set('row', $row);
		$this->render('/books/print_new');
	}
	
	/**
	 * Печать новых поступлений
	 *
	 * @param srt $period код периода для печати
	 */
	function print_new($period='d'){
		$this->layout = 'ajax';
		if(!key_exists($period,$this->periods)){
			$period = 'd';
		}
		$delay = coalesce(Configure::read('opac.book.arrival_w8'),array('-30','-1','-2'));
		$this->Book->arrivals = compact('period', 'delay');
		$res = $this->Book->findAll(array('Book.Categ'=>$this->categs));			
		$sources = Set::combine($res, '{n}.detail_url.IdNotice', '{n}.detail_url.IdNotice','{n}.detail_url.Source');
		$res = $this->requestAction(array('controller' => 'Books', 'action' => 'print_books'), array('return', 'form' => $sources));
		$this->set('res',$res);
	}
	
	/**
	 * Проверка имеющихся у пользователя подписок (абонементов), 
	 * т.е. записан ли он в библиотеку и действительна(ы) ли его запись(и)
	 * 
	 * @todo см код функции
	 * 
	 * @param unknown_type $IdAnnexe
	 * @param unknown_type $IdUsager
	 * @return boolean
	 */
	function _checkUserSouscriptions($IdUsager, $IdAnnexe) {
		// Загрузка данных о действующих абонементах и установленных тарифах для выдачи книг
		// 
		$conditions = array('User.IdUsager ='=>$IdUsager);
		$active = array();
		$active = array('UsagerSouscription.DateDebut <= getdate()', 'UsagerSouscription.DateFin >= getdate()');
		$forCurrentAnnexe = array('OR'=>array('Tarif.IdAnnexe =' => $IdAnnexe, 'Tarif.IdAnnexe IS NULL'));
		$contain = array('Souscription'=>array('Tarif'=>array('conditions'=>$forCurrentAnnexe), 'conditions'=>$active));
		
		// Получение выборки
		$user_sous = $this->User->find('first', compact('conditions','contain'));
		if (empty($user_sous['Souscription'])) {
			$message = sprintf('<strong> %s </strong>',__('Внимание:',true));
			$message.=__('У Вас отсутствует действующий абонемент.', true);
			$this->Session->setFlash($message);
		}
		
		if (empty($user_sous)) {
			$this->setErrorFlash(__('Пользователь не найден.', true));
			return false;
		}
		
		/** 
		 * @todo Генерация предупреждений пользователю ("Выдача может быть невозможна") - не должны затираться результатом
		 * @todo Такие бронирования должны быть специально помечены для анализа библиотекарем
		 *
		 */
		return true;
	}
	
	/**
	 * Проверка максимального числа бронирований для пользователя
	 * @todo использовать NBMAXDOCUSAGER из таблицы DetailParametre или другой параметр
	 * 
	 * @return boolean
	 */
	function _checkMaxReservations($IdUsager) {
		$global_max_r = 5;
		$user_max_r   = $this->User->field('NbreResaMaxi', array($this->User->primaryKey => $IdUsager));
		
		/** @var integer $max_r максимальное количество бронирований для данного пользователя **/
		$max_r = coalesce($user_max_r, $global_max_r);
		
		// Read current reservations count from DB
		//
		$fields     = array('COUNT(*) AS count');
		// активные для данного пользователя, не аннулированные и не удовлетворенные
		$conditions = array('IdUsager'=>$IdUsager, 'Annulee'=>null, 'Satisfaite'=>null); 
		$current_r = $this->Reservation->find('first', compact('fields','conditions'));
		if (!empty($current_r) && $current_r[0]['count'] >= $max_r) {
			$this->setErrorFlash(__('Невозможно забронировать документ: превышено максимальное число бронирований для пользователя', true));
			return false;
		}
		return true;
	}
	
	/**
	 * Проверка, что экземпляр этого издания не забронирован уже и не выдан уже даному читателю
	 * @todo добавить проверку на то, что экземпляр издания не выдан уже пользователю
	 *
	 * @param unknown_type $IdUsager
	 * @param unknown_type $IdNotice
	 */
	function _checkNotReservedOrPreted($IdUsager, $IdNotice) {
		// Проверка бронирований
		//
		$fields     = array('COUNT(*) AS count');
		// активные для данного пользователя, не аннулированные и не удовлетворенные
		$conditions = array('IdUsager'=>$IdUsager, 'Annulee'=>null, 'Satisfaite'=>null, "{$this->Reservation->useTable}.IdNotice"=>$IdNotice); 
		$current_r  = $this->Reservation->find('first', compact('fields','conditions'));
				
		if (!empty($current_r) && $current_r[0]['count'] > 0) {
			$this->setErrorFlash(__('Невозможно забронировать документ: документ уже забронирован для Вас', true));
			return false;
		}		
		
		$fields = $conditions = $contain = $group = null;
		// Проверка выдач
		//
		$fields     = array('COUNT(*) AS count');
		$contain    = array('Exemplaire'=>array('Notice'));   
		$group      = array('Exemplaire.IdExemplaire');
		// активные для данного пользователя
		$conditions = array('IdUsager'=>$IdUsager, 'Exemplaire.IdNotice'=>$IdNotice); 
		
		$current_r  = $this->Pret->find('first', compact('fields', 'conditions', 'contain', 'group'));
		$this->set('reservation', $current_r);
		
		if (!empty($current_r) && $current_r[0]['count'] > 0) {
			$this->setErrorFlash(__('Невозможно забронировать документ: документ уже выдан Вам', true));
			return false;
		}
		
		return true;
	}
	
	/**
	 * Проверка задолженностей и штрафов
	 * 
	 * @example Укажите непустой $IdAnnexe (!= null) если проверятся задолженность только в одном филиале,
	 *          иначе проверяется задолженность по всем филиалам
	 *
	 * @param integer $IdUsager 
	 * @param integer $IdAnnexe
	 * @return boolena
	 */
	function _checkUserNoPenalsNoHoldOver($IdUsager, $IdAnnexe) {
		// Проверка задолженности
		//
		$holds = $this->Pret->getHoldOverExemplaires($IdUsager);
		
		if (!empty($holds)) {
			$this->setErrorFlash(__('Невозможно забронировать документ: имеется задолженность', true));
			return false;
		}
		
		
		// Проверка назначенных штрафов
		//
		$susp_debut = $this->Auth->user('DateDebutSuspension');
		$susp_end   = $this->Auth->user('DateFinSuspension');
		if (empty($susp_debut) || empty($susp_end)) {
			return true;			
			$message = sprintf($message, date('d-M-Y', $susp_end));
			$this->setErrorFlash($message);
			return false;
		}
		
		return true;
	}
	
	function numberOfExemplaires($IdNotice, $source = "default"){
		// @todo check availibility of exemplaires, and do reservation for first availible (use autoinc)
		$ex = $this->Exemplaire->alias;
		$fields = array("$ex.IdExemplaire", "$ex.CodbarE");
		$contain = array('Statut');
		$conditions = array("$ex.IdNotice"=>$IdNotice,
		  "$ex.IdExemplaire NOT IN (SELECT IdExemplaire FROM EXEMPLAIRE_RESA WHERE IdReservation IS NOT NULL)",
		  "$ex.IdExemplaire NOT IN (SELECT IdExemplaire FROM PRET)",
		  "Statut.Pretable"=>1
		  //"Statut.Reservable"=>1
		  );
		
		  $exemp_availible = $this->Exemplaire->find('all', compact('fields','contain','conditions')); 
		
		if (key_exists('return', $this->params)) {
			return count($exemp_availible);
		}
		  
		$this->set('numberPretable', count($exemp_availible));
		return;
	}
	
	function _addReservationToAutorecordArmoire($AddReservationReq, $IdNotice, $IdAnnexe, $IdUsager){
		$userInfo = $this->User->getUserInfo($IdUsager);
		$userCodbarU = intval($userInfo[0]['codbarU']);
		$minCodbar = $this->Uruser->minCodbar;
		$maxCodbar = $this->Uruser->maxCodbar;
		
		// Если читатель имеет штрихкод, лежащий в диаппазоне указанном в плагине "Uregistration",
		// то броникование происходит на полку "Бронеполка Самозаписи" на срок 24 часа (значения по умолчанию)
		// (имя полки и кол-во часов бронирования можно передать в хранимую процедуру "CreateArmoire")
		if($userCodbarU >= $minCodbar && $userCodbarU <= $maxCodbar){
			
			$exemplaire = $this->Exemplaire->findUnreservedExemplaire($IdNotice, $IdAnnexe);
			
			$AddReservationReqURPlugin =
				"
				DECLARE	@ArmoireID int

				EXEC [dbo].[CreateArmoire] @ArmoireID = @ArmoireID OUTPUT
				
				DECLARE @IdReservResa INT
				SET @IdReservResa = (SELECT max(IdExemplaireResa)+1 FROM EXEMPLAIRE_RESA)
				UPDATE COMPTEUR SET ValCompteur = @IdReservResa WHERE IdCompteur = 'EXEMPLAIRE_RESA'
				INSERT EXEMPLAIRE_RESA (IdExemplaireResa, IdArmoire, IdExemplaire, IdReservation, DateEntree, DateSortie, DateButoire, Imprime)
				VALUES (@IdReservResa, @ArmoireID, {$exemplaire[0][0][idexemplaire]}, @IdReservation, getdate(), null, DATEADD(hour, 24, getdate()) , 1)
				";
			$AddReservationReq .= $AddReservationReqURPlugin;
		}
		return $AddReservationReq;
	}

}
?>