<?php

require_once('widgets.inc.php');

/**
 * Контроллер для работы с моделью Paramopac,
 * содержащей настройки OPAC 
 * @author Alex
 */ 
class EditeursController extends WidgetsController{

	var $name = 'Editeurs';
	var $pageTitle;
	var $uses = array('Editeur', 'NoticeZ', 'Book');
	var $helpers = array('Html', 'Form', 'BookHelper');
	
	var $paginate = array(
        'limit' => 15,
        'order' => array(
            'Editeur.Nom' => 'asc',
            //'Auteur.PreNom' => 'asc',
        ),
        //'recursive' => 1,        
    );
	/**
	 * Editeur model
	 * @var Editeur
	 */
	var $Editeur;
	
	/**
	 * Books model
	 * @var NoticeZ
	 */
	var $NoticeZ;
	
	/**
	 * Books model
	 * @var Book
	 */
	var $Book;
	
	/**
	 * Books model
	 * @var Collection
	 */
	var $Collection;
	
	function beforeFilter() {
		parent::beforeFilter();
		$this->Auth->allow('*');
	}
	
	function beforeRender() {
		parent::beforeRender();
		$this->pageTitle = __('Справочник издательств',true);
		$default = array('Editeur.Nom'=>'');
		$request = array_merge($default, $this->params['named']);
		if($this->data){
			$request = array_merge($request, Set::flatten($this->data));
		}
		$this->set('filterValue', str_replace('%','*',$request['Editeur.Nom']));
	}

	function index($source = 'default', $remote = false) {
		$fields = array($this->Editeur->primaryKey, 'Nom', 'Ville');
		$this->paginate['fields'] = $fields;
		
		// Для отображения текущего фильтра на форме 
		// будет автоматически установлена  переменная filter для View
		//
		$default_filter = array('Editeur'=>array('Nom'=>''));
		$filter = array_merge($default_filter, $this->Filter->filterOptions);
		
		$filter_form_settings[0]['Header'] = __('Введите часть названия издательства: ',true);
		$filter_form_settings[0]['Model']  = 'Editeur';
		$filter_form_settings[0]['Fields']['Nom'] = array('label'=>'', 'type'=>'string');
					
		$this->set('filter_form_settings', $filter_form_settings);
		
		$this->Editeur->bind('Collection', array('type'=>'hasMany', 'foreignKey'=>'IdEditeur', 
		'fields' => array('COUNT(DISTINCT Collection.IdEditeur) AS total_count'),
		'group'  => array('IdEditeur')));
		
		$res = $this->paginate($this->Editeur, $this->Filter->filter);
		
		$ids = Set::extract('{n}.Editeur.IdEditeur', $res);
		$noticeCount = $this->Editeur->getNoticeCount($ids);
		
		$this->set('filter', $filter);
		$this->set('noticeCount', $noticeCount);
		$this->set('editeurs', $res);
		$this->set('idField', $this->Editeur->primaryKey);
		$this->set('source', $source);
	}
	
	function view($id) {
		$res = $this->Editeur->read(null, $id);
		$this->set('editeur', $res);
		$sortBy = array(
			'title' => 'Book.Titre',
			'date' => 'Book.IdNotice',
			'default' => 'Book.IdNotice',
		);
		$sort_name = $this->passedArgs['order'];
		$sort = 'Book.IdNotice';
		if(!empty($sort_name)) $sort = (key_exists($sort_name,$sortBy)) ? $sortBy[$sort_name] : $sortBy['default'];
		unset($this->passedArgs['order']);
		// load books 
		//
		$conditions = array('NoticeZ.IdZone'=>'210', 'OR'=>array('NoticeZ.sZone'=>array('c')), 'NoticeZ.ValeurN'=>$id);
		$recursive  = 0;
		$fields     = array('NoticeZ.IdNotice');
		$book_ids = $this->NoticeZ->find('all', compact('conditions', 'fields', 'recursive'));
		$book_ids = Set::extract('{n}.NoticeZ.IdNotice', $book_ids);
		
		if (count($book_ids) == 0) return;

		$this->Book->recursive = 1;
		$this->paginate['Book'] = array('order'=>array($sort), 'limit'=>5);
		$this->paginate['Book']['contain'] = array('NoticeZ','ExemplaireStat','Exemplaire'=>a('Statut'),'Rating');
		$this->set('books', $this->paginate($this->Book, array('Book.IdNotice'=>$book_ids)));
		$ids_print = $this->params['form']['ids_print'];
		if($ids_print !=''){
			$this->set('ids_print',$ids_print);
		}
		
		if($this->RequestHandler->isAjax()) { 
			$this->viewPath = 'books';
			$this->set('isAjax', true);
			$this->render('books_list_ajax'); 
		};

	}
}
?>