<?php
class Matiere extends AppModel {

	var $name = 'Matiere';
	var $useTable = 'Matiere';
	var $actsAs = array('Containable'); 
	
	var $hasMany = array(
	'NoticeZ' => array(
		'className' => 'NoticeZ',
		'foreignKey' => 'ValeurN',
		'conditions' => array('NoticeZ.sZone' => 'a', 'NoticeZ.IdZone' => 'R600'),
		'fields'     => array('COUNT(DISTINCT NoticeZ.IdNotice) AS total_count'),
		'group'      => array('ValeurN')
		)
	);
	
	var $belongsTo = array(
		'Thesaurus' => array(
		'className' => 'Thesaurus',
		'foreignKey'    => 'IdThesaurus'
		),
		'RetenueMatiere' => array(
	    'className' => 'Matiere',
	    'foreignKey'    => 'IdRetenue'
		)
	);
	
	var $chain;
	
	var $ac_params = array('search'=>'SujetRech', 'value'=>'Sujet', 'result'=>'SujetRech');
	
	function getAutocomplete($q, $limit, $conditions = array()){
		if(mb_strpos($q,' + ') !== false){
			$sujet = trim(mb_substr($q,mb_strrpos($q,' + ')+2));
			$parentSujet = trim(mb_substr($q,0,mb_strrpos($q,' + ')));
			$parentIds = $this->getIdMatiereRecursive($parentSujet);
			$conditions = array('Matiere.IdPere'=>$parentIds);
			$data = parent::getAutocomplete($sujet, $limit, $conditions);
			foreach ($data as $k=>$data_){
				$data[$k]['result'] = str_replace('%','*',$parentSujet).' + '.$data_['result'];
				$data[$k]['value'] = str_replace('%','*',$parentSujet).' + '.$data_['value'];
			}
			return $data;
		}else{
			return parent::getAutocomplete($q, $limit, $conditions = array());
		}
	}
	
	function beforeFind($queryData) {
		parent::beforeFind();
		$opac_config = Configure::read('opac_sources');
		$id_thesaurus_disc = $opac_config->GetParam('default', 'KNIGOB_ID_THESAURUS_DISC');
		
		// Необходимо, чтобы при использовании типа поиска list
		// корректно использовался вычисляемый столбец
		//
		if(key_exists('list', $queryData)) {
			$path = explode('AS', $queryData['list']['valuePath']);
			$queryData['list']['valuePath'] = '{n}.0.'.trim($path[1]);
		}		
		
		// Работа со звездочками и ускорение фильтрации
		//
		foreach ($queryData['conditions'] as $key => $value) {
			$queryData['conditions'][$key] = str_replace(array('*','Sujet '), array('%', 'SujetRech '), $value);
		}
		
		return $queryData;
	}
	
	/**
	 * Возвращает массив рубрик для текущей, начиная с хвоста.
	 * @todo можно вынести смену рекурсии и формирования условий из 
	 * 		 функции рекурсии для быстроты
	 * @param int $idMatiere - id рубрики
	 * @param string $source - источник БД
	 * @param array $chain - массив рубрик (нужен для рекурсии)
	 * @return array
	 */
	function getFullChain($idMatiere,$source='default',$chain = array()){
		$fields = array('IdPere','Sujet');
		$conditions = array('Matiere.IdMatiere'=>$idMatiere);
		
		$recursion = $this->recursive;
		$this->recursive = 0;
		
		$res = $this->find('all',compact('conditions','fields'));
		$idPere = intval($res[0]['Matiere']['IdPere']);
		
		$this->recursive = $recursion;
		
		if($idPere>0){
			$chain[] = $res[0]['Matiere'];
			$chain = $this->getFullChain($idPere,$source,$chain);
			return $chain;
		}else{
			$chain[] = $res[0]['Matiere'];
			return $chain;
		}
	}
	
	/**
	 * Возвращает двухмерный массив, в котором ключи - идентификаторы рубрик, значения - количество подрубрик
	 *
	 * @param int $id
	 * @param boolean $direct - если истина, считает только прямых потомков, иначе - всех
	 * @return array
	 */
	function get_subcount($id = null, $direct = true) {
		if ($direct) {
			return $this->find('list', array(
			  'fields' => array('Matiere.IdPere', 'COUNT(Matiere.IdMatiere) AS subcount'),
			  'conditions' => array($this->escapeField('IdPere') => $id), 
			  'group'      => 'IdPere'));
		} else {
			$base = $this->table;
			$this->setSource('LienMatiere');
			return $this->find('list', array(
			  // COUNT-1, так как LienMatiere для любой рубрики IdMatiere содержит строку со связью IdMatiere => IdMatiere
			  'fields' => array('IdMatiereParent', 'COUNT(Matiere.IdMatiereChild)-1 AS subcount'),
			  'conditions' => array('IdMatiereParent' => $id), 
			  'group'      => 'IdMatiereParent'));
		}
	}
	
	/**
	 * Функция подсчитывает количество описаний в указанной рубрике(ах)
	 * Подсчет ведется с учетом описаний в подрубриках (по умолчанию, при $recursive = true)
	 * или без учета подрубрик 
	 * 
	 * @todo без учета подрубрик
	 *
	 * @param mixed   $id - идентификатор рубрики или массив идентификаторов
	 * @param boolean $recursive - учет описаний в подрубриках при подсчете
	 * @return array
	 */
	function getNoticeCount($id, $recursive = true) {
		$habtm = array(
		'SubMatieres' => array(
			'className' => 'Matiere',
			'joinTable' => 'LienMatiere',
			'foreignKey' => 'IdMatiereParent',
			'associationForeignKey' => 'IdMatiereChild',
			'unique' => false,
			'fields' => array('IdMatiere'),
		));
		
		$this->bindModel(array('hasAndBelongsToMany'=>$habtm), true);
		
		$data = $this->find('all', array(
		'conditions' => array('IdMatiere' => $id), 
		'contain'=>array('SubMatieres'=>array('NoticeZ')),
		'fields' =>array('IdMatiere')
		));
		
		$data = Set::combine($data, 
		'/SubMatieres/IdMatiere', 
		'/SubMatieres/NoticeZ/NoticeZ/total_count', 
		'/SubMatieres/LienMatiere/IdMatiereParent');
		
		$result = array();
		foreach ($data as $key => $value) {
			$result[$key] = array_sum($value);
		}	
		
		return $result;
	}
	
	/**
	 * Возвращает id всех рубрик, удовлетворяющих условию
	 * поиска по названию вышестоящих рубрик и собственному названию 
	 *
	 * @param строка названий рубрик и подрубрик, разделённых знаком "+" 
	 * @return array id рубрик, удовлетворяющих запросу.
	 */
	function getIdMatiereRecursive($sujet/*,$thesaurus_id*/,$source='default'){
		$this->useDbConfig = $source;
		$dbo = $this->getDataSource();
		$sujet = str_replace('*',"%",$sujet);
		
		$matieres = explode(' + ',$sujet);	
		$sql = '';
		foreach ($matieres as $matiere){
			$req = $this->_getSubRequest($matiere,$dbo/*,$thesaurus_id*/);
			$sql = (!$sql) ? $req : $req." AND Matiere.IdPere in ($sql)" ;
		}
		// возвращаем id
		return Set::extract('{n}.0.IdMatiere',$this->query($sql));
	}
	
	// формирует подзапрос для getIdMatiereRecursive
	function  _getSubRequest($matiere,$dbo/*,$thesaurus_id*/){
		$param = array(
			'conditions' => array(
				'Matiere.Sujet LIKE' => $matiere, 
				/*'Matiere.IdThesaurus'=>$thesaurus_id*/),
			'fields' => array('Matiere.IdMatiere'),
			'table' => $dbo->fullTableName($this),
			'alias' => 'Matiere',
		);
		return $dbo->buildStatement($param,$this);
	}

}
?>