<?php
class Rating extends AppModel {

	var $name = 'Rating';
	var $primaryKey = 'id'; // Cake standart!

	//The Associations below have been created with all possible keys, those that are not needed can be removed
	var $belongsTo = array(
		'Notice' => array(
			'className' => 'Notice',
			'foreignKey' => 'notice_id',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		)
	);

	var $hasMany = array(
		'Ratelog' => array(
			'className' => 'Ratelog',
			'foreignKey' => 'rating_id',
			'dependent' => false,
			'conditions' => '',
			'fields' => '',
			'order' => '',
			'limit' => '',
			'offset' => '',
			'exclusive' => '',
			'finderQuery' => '',
			'counterQuery' => ''
		)
	);
	
	/**
	 * Функция прибавляет заданный голос к имеющимся (если имеются)
	 * @example $vote = array('notice_id'=>31, 'rating'=>5, 'votes'=>1, 'numVotes'=>1, 'viewed'=>1); 
	 * @example если хотите, вы можете передать только оценку 'rating' и идентификатор издания 'notice_id'
	 * @param array $vote
	 */
	function inc($vote) {
		// rating is null by default so if it is null in $rec we will get error from DBMS
		$default_vote = array('rating'=>null, 'votes'=>null, 'numVotes'=>1, 'viewed'=>1);
		$vote = array_merge($default_vote, $vote);
		
		if (!key_exists('notice_id', $vote)) return false;
		$notice_id = intval($vote['notice_id']);
		if (empty($notice_id)) return false;
		
		$db =& ConnectionManager::getDataSource($this->useDbConfig);
		// **************** BEGIN TRANSACTION ************* //
		$db->begin($this);
			
		$stored_data = $this->find('first', array(
				'conditions' => array($this->alias . '.notice_id' => $notice_id)
			));
		
		$rec = array();
		if (empty($stored_data)) {
			$rec = $vote;
			$rec['votes'] = $vote['rating']; // Общий бал установим равный поданному голосу
			$this->create();
			$result = $this->save($rec);
			
			// *************** COMMIT INSERT ************ //
			$db->commit($this);
			return $result;
		} else {
			$old_rec = $stored_data[$this->alias];
			$new_votes    = $old_rec['votes'] + $vote['rating'];
			$new_numVotes = $old_rec['numVotes'] + $vote['numVotes'];
			$new_rating   = round(($new_votes / $new_numVotes), 2);
			
			$rec=array($this->primaryKey => $old_rec[$this->primaryKey],
			           'notice_id' => $old_rec['notice_id'],
			           'votes'     => $new_votes,
			           'numVotes'  => $new_numVotes,
			           'viewed'    => $old_rec['viewed']+ $vote['viewed'],
			           'rating'    => $new_rating
			           );
			$result = $this->save($rec);
			
			// *************** COMMIT UPDATE ************//
			$db->commit($this);
			return $result;
		}
		
	}

}
?>