<?php
class Uruser extends UregistrationAppModel{
    
    var $name       = 'Uruser';
    var $useTable   = 'Usager';
    var $primaryKey = 'IdUsager';

    var $hasMany = array(
    	'UserAddress' => array(
    		'className' => 'Uregistration.UserAddress',
    		'foreignKey' => 'IdUsager'
    	)
    );
    
    /**
     * Максимальный номер штрихкода пользователя для диаппазона
     *
     * @var int
     */
    var $minCodbar = 500001;
    
    /**
     * Минимальный номер штрихкода пользователя для диаппазона
     *
     * @var int
     */
    var $maxCodbar = 500100;
    
    /**
     * Максимальное колличество резерваций для пользователя
     *
     * @var int
     */
    var $maxReservations = 5;
    
    /**
     * Правила валидации для формы самозаписи
     *
     * @var array
     */
    var $validate = array();
    
    function __construct(){
    	parent::__construct();
    	
    	$this->validate = array(
            'Nom' => array(
                'rule' => VALID_NOT_EMPTY,//alphaNumeric
                'required' => true,
                'message'  => __('Поле не должно быть пустым.',true),
            ),
            'Prenom' => array(
                'rule' => VALID_NOT_EMPTY,//array('custom', '/^[a-zA-Z ]*$/i'),
                'required' => true,
                'message'  => __('Поле не должно быть пустым.',true),
            ),
            'Notes' => array(
            	'rule' => VALID_NOT_EMPTY,
            	'required' => true,
            	// message задается в index.ctp
            )
        );
    	
    }
    
    function getMinUserCodbar(){
        $genCodbars = range($this->minCodbar, $this->maxCodbar);
        $conditions = array(
            'fields' => array('CodbarU'),
            'conditions' => array("CodbarU between $this->minCodbar and $this->maxCodbar"),
        );
        $codbars = $this->find('all',$conditions);
        $codbars = array_map(array('Uruser','__filter'),$codbars);
        $diffCodbars = array_diff($genCodbars, $codbars);
        $firstCodbar = array_shift($diffCodbars);
        return $firstCodbar;
    }
    
    function getMaxUserId(){
        $id = $this->query('SELECT max(IdUsager) as idUsager FROM USAGER');
        $id = Set::extract('{n}.0.idUsager',$id);
        return $id[0]+1;
    }
    
    function __filter($element){
        return intval($element["Uruser"]["CodbarU"]);
    }
    
    function addNewUser($data, $userId){
    	
    	$minCodbar = $this->getMinUserCodbar();
    	if(!$minCodbar)
    		return __('Регистрация невозможна. Достигнуто максимальное число пользователей.', true);
    	
    	$coincidenced = $this->findNomPrenomCoincidence($data['Uruser']['Nom'], $data['Uruser']['Prenom']);
    	if($coincidenced)
    		return __('Пользователь с такими именем и фамилией уже зарегистрирован.', true);
    	
    	$currTime = date('Y-m-d H:i:s',time());
    	
    	$additionalInfo = array(
    		'IdUsager' => $userId,
    		'MotPasse' => $this->getUserPass($data['Uruser']['Prenom']),
    		'CodbarU' => $minCodbar,
    		'DateCreation' => $currTime,
    		'DateInscription' => $currTime,
    		'NbreResaMaxi' => $this->maxReservations,
    		'IdSexe' => 0,
    		'IdProfession' => 0,
    		'IdCategorie' => 0,
    		'IdQuartier' => 0,
    		'IdMotifSuspension' => 0,
    		'IdResponsable' => 0,
    	);
    	
    	$data['Uruser'] = array_merge($data['Uruser'], $additionalInfo);
    	
    	$fields = $this->save($data);
    	
    	$this->query("UPDATE COMPTEUR SET ValCompteur = (SELECT max(IdUsager) FROM USAGER) WHERE IdCompteur = N'USAGER'");
    	
    	return $fields;
    }
    
    function getUserPass($userNom){
    	return $userNom;
    }
    
    /**
     * Поиск совпадений имени и фамилии, если есть совпадения, возвращает true
     *
     * @param string $nom
     * @param string $prenom
     * @return array
     */
    function findNomPrenomCoincidence($nom,$prenom){
    	
    	$nom = mb_strtolower($nom, 'utf-8');
    	$prenom = mb_strtolower($prenom, 'utf-8');

        $nomCoincidence = $this->find('all',array(
            'fields' => array('Nom'),
            'conditions' => array("LOWER(Nom) = N'$nom'"),
        ));
        

        $prenomCoincidence = $this->find('all',array(
            'fields' => array('Prenom'),
            'conditions' => array("LOWER(Prenom) = N'$prenom'"),
        ));
        
        
        if($nomCoincidence && $prenomCoincidence){
        	$nomIds    = Set::extract('{n}.Uruser.IdUsager',$nomCoincidence);
        	$prenomIds = Set::extract('{n}.Uruser.IdUsager',$prenomCoincidence);
        	
        	$merged = array_merge($nomIds, $prenomIds);
        	$beforeCount = count($merged);
        	$unique = array_unique($merged);
        	$afterCount = count($unique);
        	
        	if($beforeCount != $afterCount)
        		return true;
        }
        return false;
    }
}
?>
