<?php
/**
 * Глобальное хранилище настроек поисковых полей по умоланию, 
 * соответствий просмотров, списка полнотекстовых зон и т.п.
 *
 */

class FieldOptions {
	/**
	 * Соответствия операций в GET запросе и операций сравнения.
	 *
	 * @var array
	 */
	var $aOperations = array ('SE' => '>=', //Supér. ou égal
	'IE' => '>=', 'S' => '>', 'I' => '<', 'N' => '!=', 'DEFAULT' => '=' );
	
	/**
	 * Массив определяет поля и подполя, для которых используется полнотекстовый поиск
	 * 
	 * @var array
	 */
	var $aFullTextFields = array ('200' => array ('a', 'd', 'e'), '503' => array ('a' ) );
	var $FullTextEnabled = true;
	var $aViews = array ();

	/**
	 * Позволяет настроить просмотр для поиска по определенному полю и подполю
	 *
	 * @example addView('view_200ad_503a', '200', 'a' ); 
	 *  
	 * @param string $sView  - имя используемого просмотра
	 * @param string $IdZone - поле
	 * @param string $sZone  - подполе
	 */
	function addView($sView, $IdZone, $sZone) {
		$aView = &$this->aViews [$sView];
		if (! is_array ( $aView )) {
			$aView = array ();
		}
		if (! empty ( $IdZone ) || ! empty ( $sZone )) {
			$aView [] = $IdZone . '_' . $sZone;
		}
	}

	function getView($IdZone, $sZone) {

		foreach ( $this->aViews as $aView ) {
			$result = array_search ( $IdZone . '_' . $sZone, $aView, true );
			if ($result === false)
			continue;
			return array_search ( $aView, $this->aViews );
		}

		return $result;
	}
	
	function addField($field, $subField,  $IdZone, $sZone ){
		$aField = &$this->aField [$field];
		if (! is_array ( $aField )) {
			$aField = array ();
		}
		if (! empty ( $IdZone ) || ! empty ( $sZone )) {
			$aField[$subField] [] = $IdZone . '_' . $sZone;
		}
	}
	
	function getField($IdZone, $sZone) {

		foreach ( $this->aField as $table=>$aField ) {
			foreach ($aField as $column=>$subField){
				$result = array_search ( $IdZone . '_' . $sZone, $subField, true );
				if ($result === false)
				continue;
				return array('table'=>$table, 'column'=> $column);
			}
		}

		return ($result) ? $result : false;
	}

	/**
	 * Singleton helper. Call staticaly
	 * 
	 * @example  $oOptions = FieldOptions::getInstance()
	 *  
	 * @return FieldOptions
	 */
	function &getInstance() // this implements the 'singleton' design pattern.
	{
		static $instance;
		if (! isset ( $instance )) {
			$c = __CLASS__;
			$instance = new $c ( );
		} // if
		return $instance;
	} // getInstance

	function FieldOptions() {
		
	}
	
	function InitConfig() {
		if(low(Configure::read('opac.search.full_title')) == 'on'){
			$this->addField ('NOTICE', 'TitreRech', '200', 'a');
		}else{
			$this->addView ( 'view_NOTICE_200ade_503a', '200', 'a' ); // Заглавие
		}
		$this->addView ( 'view_NOTICE_200ade_503a', '200', 'd' ); //
		$this->addView ( 'view_NOTICE_200ade_503a', '200', 'e' );
		$this->addView ( 'view_NOTICE_200ade_503a', '503', 'a' );
		$this->addView ( 'view_200fg', '200', 'f' ); // Первый автор
		$this->addView ( 'view_200fg', '200', 'g' ); // Другие авторы
		$this->addView ( 'view_210c', '210', 'c' );  // Издательство
		$this->addView ( 'view_210d', '210', 'd' );  // Дата
		$this->addView ( 'view_NOTICE_225a', '225', 'a' );  // Серия
		$this->addView ( 'view_NOTICE_225ai', '225', 'i' );  // Подсерия
		//$this->addView ( 'view_NOTICE_675a_686a', '675', 'a' ); // УДК
		switch (Configure::read('opac.search.index_udc')){
			case 'COTE':
				$this->addView ('view_EXEMPLAIRE', '675', 'a' );//Полочный индекс
				break;
			case 'UDC':
			case 'BBK':
			default:
				$this->addView ('view_NOTICE_675a_686a', '675', 'a' ); // УДК
		}
		$this->addView ( 'view_NOTICE_675a_686a', '686', 'a' ); // ББК
		//$this->addView ( 'view_225', '225', 'a' ); //Серия
		//$this->addView ( 'view_210d', '210', 'd' );//Дата
		$this->addView ( 'view_R600a', 'r600', 'a' );//Рубрика
		/*$this->addView ( "dbo.NOTICE_Z INNER JOIN dbo.LienMatiere 
		ON (LienMatiere.IdMatiereChild=dbo.NOTICE_Z.ValeurN) AND (dbo.NOTICE_Z.IdZone = 'R600') AND (dbo.NOTICE_Z.sZone = 'a')
			INNER JOIN dbo.MATIERE ON LienMatiere.IdMatiereChild=MATIERE.IdMatiere", 'sujet', '' );//Рубрика*/
		//Нумерация
		$this->addView ( 'view_EXEMPLAIRE', 'cote', '' );//Полочный индекс
		//Жанр
		$this->addView ( 'view_EXEMPLAIRE', 'codbare', '' );//Штрих-код
		$this->addView ( 'view_EXEMPLAIRE', 'inventaire', '' );//Инв. номер
		$this->addView ( 'view_NOTICE_CATEG', 'idcategnotice', '' );// тип док-та
		$this->addView ( 'NOTICE', 'categ', '' );// тип док-та
		//Язык документа
		//Место издания
	}
}


?>
